﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"
#include "..\IMatrixDisplay.h"

namespace NonAffineStretch
{
    class NonAffineStretchRenderer
    {
    public:
        NonAffineStretchRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources,
                                 IMatrixDisplay^ matrixDisplay);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Update(DX::StepTimer const& timer);
        bool Render();

        // Asynchronous pointer events
        void OnPointerPressed(unsigned int id, D2D1_POINT_2F pointerPos);
        void OnPointerMoved(unsigned int id, D2D1_POINT_2F pointerPos);
        void OnPointerReleased(unsigned int id, D2D1_POINT_2F pointerPos);

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;

        IMatrixDisplay^                      m_matrixDisplay;

        bool m_needsRedraw;
        D2D1::Matrix3x2F m_displayTransform;
        D2D1::Matrix3x2F m_inverseDisplayTransform;

        // Bitmap and matrix
        Microsoft::WRL::ComPtr<ID2D1Bitmap1>    m_bitmap;
        D2D1_MATRIX_4X4_F                       m_matrix;

        // Thumbs
        static const int m_thumbRadius = 20;

        struct Thumb
        {
            D2D1_POINT_2F center;
            bool isTracking;
        };

        std::array<Thumb, 4> m_thumbs;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_thumbBrush;

        // Pointer tracking
        struct PointerInfo
        {
            int           thumbIndex;
            D2D1_POINT_2F lastPointerPosition;
        };

        std::map<unsigned int, PointerInfo> pointerTracker;
        void MoveThumb(PointerInfo& pointerInfo, D2D1_POINT_2F pointerPos);

        inline bool HitTest(D2D1_POINT_2F pt1, D2D1_POINT_2F pt2, float offset)
        {
            return std::abs(pt1.x - pt2.x) < offset && std::abs(pt1.y - pt2.y) < offset;
        }

        void CalculateNewTransform();
        D2D1_MATRIX_4X4_F CalculateNewTransform(D2D1_SIZE_F size, 
                                                D2D1_POINT_2F ptUL, 
                                                D2D1_POINT_2F ptUR,
                                                D2D1_POINT_2F ptLL, 
                                                D2D1_POINT_2F ptLR);
    };
}