﻿#pragma once

#include "Common\StepTimer.h"
#include "Common\DeviceResources.h"
#include "Content\NonAffineStretchRenderer.h"
#include "IMatrixDisplay.h"

// Renders Direct2D and 3D content on the screen.
namespace NonAffineStretch
{
    class NonAffineStretchMain : public DX::IDeviceNotify
    {
    public:
        NonAffineStretchMain(const std::shared_ptr<DX::DeviceResources>& deviceResources,
                             IMatrixDisplay^ matrixDisplay);
        ~NonAffineStretchMain();
        void CreateWindowSizeDependentResources();
        void StartRenderLoop();
        void StopRenderLoop();
        Concurrency::critical_section& GetCriticalSection() { return m_criticalSection; }

        // IDeviceNotify
        virtual void OnDeviceLost();
        virtual void OnDeviceRestored();

        // Asynchronous pointer events
        void OnPointerPressed(unsigned int id, Windows::Foundation::Point point);
        void OnPointerMoved(unsigned int id, Windows::Foundation::Point point);
        void OnPointerReleased(unsigned int id, Windows::Foundation::Point point);

    private:
        void ProcessInput();
        void Update();
        bool Render();

        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Content renderer
        std::unique_ptr<NonAffineStretchRenderer> m_nonAffineStretchRenderer;

        Windows::Foundation::IAsyncAction^ m_renderLoopWorker;
        Concurrency::critical_section m_criticalSection;

        // Rendering loop timer.
        DX::StepTimer m_timer;

        // Track current input pointer position.
        float m_pointerLocationX;
    };
}